/* Window Picker */

$window_picker_spacing: $base_padding; // 6px
$window_picker_padding: $base_padding * 2; // 12px

$window_thumbnail_label_color: transparentize($system_bg_color, 0.4);

$window_close_button_color: transparentize(lighten($system_bg_color, 7%), .02);
$window_close_button_size: 30px;
$window_close_button_padding: 3px;

// Window picker
.window-picker {
  // Space between window thumbnails
  spacing: $window_picker_spacing;
}

// Window titles
.window-caption {
  @extend %tooltip;
}

// Close button
.window-close {
  background-color: $window_close_button_color;
  color: $system_fg_color;
  border-radius: 99px;
  box-shadow: 0 2px 4px 0 $shadow_color;
  padding: $window_close_button_padding;
  height: $window_close_button_size;
  width: $window_close_button_size;
  transition-duration: 100ms;

  & StIcon { icon-size: 24px; } // uses non standard icon size

  &:hover {
    background-color: lighten($window_close_button_color, 7%);
  }

  &:active {
    background-color: lighten($window_close_button_color, 13%);
  }

  @if $is_highcontrast {
    border:2px solid $outer_borders_color;
  }
}

.workspace-background {
  // keep in sync with BACKGROUND_CORNER_RADIUS_PIXELS in workspace.js
  border-radius: 30px;
  background-color: $invisible_occluded_bg_color;
  box-shadow: 0 4px 16px 4px $shadow_color;
}
