/* Buttons */

.button {
  @extend %button; // that's it
  min-height: 22px;
}

.icon-button {
  @extend .button; // same style as buttons

  border-radius: 99px;
  padding: $base_padding*2;
  min-height: 16px;

  StIcon {
    icon-size: $base_icon_size;
    -st-icon-style: symbolic;
  }
}
