/* OSD */

$osd_levelbar_height:6px;

.osd-window {
  @extend %osd_panel;
  @extend %heading;
  text-align: center;
  font-weight: bold;
  spacing: $base_padding * 2; // 12px
  padding: $base_padding * 2 $base_padding * 3;
  & > * { spacing: 8px; }
  margin-bottom: 4em;

  StIcon { icon-size:$large_icon_size;}

  StLabel {
    &:ltr { margin-right: 6px; }
    &:rtl { margin-left: 6px; }
  }

  .level {
    margin-bottom: 4px;
    &:first-child { margin-bottom: 0px; }

    height: $osd_levelbar_height;
    min-width:$base_icon_size * 10;
    -barlevel-height: $osd_levelbar_height;
    -barlevel-background-color: transparentize($osd_fg_color, if($variant=='light', 0.7, 0.9));
    -barlevel-active-background-color: $osd_fg_color;
    -barlevel-overdrive-color: $destructive_color;
    -barlevel-overdrive-separator-width: 3px;
    &:ltr { margin-right: 6px; }
    &:rtl { margin-left: 6px; }
  }
}

// Monitor number label
.osd-monitor-label { 
  @extend %osd_panel;
  border-radius: $modal_radius;
  font-size: 3em;
  font-weight: bold;
  margin: $base_margin * 3;
  text-align: center;
  min-width: 1.3em;
}

/* Pad OSD */
.pad-osd-window {
  padding: 32px;
  background-color: transparentize(#000, 0.2);

  .pad-osd-title-box { spacing: 12px; }
  .pad-osd-title-menu-box { spacing: 6px; }
}

.combo-box-label {
  width: 15em;
}

.resize-popup {
  @extend %osd_panel;
}
