// Search entry
.search-entry {
  border-radius: 99px;
  margin-top: $base_padding*2;
  margin-bottom: $base_padding;
  width: 24em;

  @extend %system_entry;

  .search-entry-icon {
    icon-size: $base_icon_size;
    margin-top: 2px; // center vertically
    padding: 0 4px;
  }
}
